package com.example.aped;

import com.example.aped.utils.XMLHandler;

import org.junit.Test;
import org.xml.sax.SAXException;

import java.io.File;
import java.io.IOException;
import java.util.Dictionary;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;

import static org.junit.Assert.*;

public class XMLHandlerUnitTest {
    private String xmlPath="src/main/assets/XML/Test.xml";
    private File xmlFile = new File(xmlPath);

    @Test
    public void TestFiles_AreValid(){
        try{
            XMLHandler xmlHandler = new XMLHandler(xmlFile);
        }catch(IOException | ParserConfigurationException | SAXException e){
            System.out.println("XMLHandler failed");
            assert(false);
        }catch(VerifyError e){
            System.out.println("XML not valid");
            assert(false);
        }
    }

    @Test
    public void Test_getDeviceNames(){
        try{
            XMLHandler xmlHandler = new XMLHandler(xmlFile);
            List<String> deviceNames = xmlHandler.getDeviceNames();
            assertArrayEquals(new String[]{"example","sensorarray"},deviceNames.toArray());
        }catch(IOException | ParserConfigurationException | SAXException e){
            System.out.println("XMLHandler failed");
            assert(false);
        }
    }

    @Test
    public void TestInput_SimpleValueInfo(){
        try{
            XMLHandler xmlHandler = new XMLHandler(xmlFile);
            Dictionary<String, Object> valueInfo = xmlHandler.getValueInfo("example");
            assertEquals("{factor=1.0, type=boolean, unit=, offset=0.0}",valueInfo.toString());
        }catch(IOException | ParserConfigurationException | SAXException e){
            System.out.println("XMLHandler failed");
            assert(false);
        }
    }

    @Test
    public void TestInput_ComplexValueInfo(){
        try{
            XMLHandler xmlHandler = new XMLHandler(xmlFile);
            Dictionary<String, Object> valueInfo = xmlHandler.getValueInfo("sensorarray");
            assertEquals("{factor=2.5, type=int, unit=°C, offset=1.2}",valueInfo.toString());
        }catch(IOException | ParserConfigurationException | SAXException e){
            System.out.println("XMLHandler failed");
            assert(false);
        }
    }

    @Test
    public void TestInput_SimplePort(){
        try{
            XMLHandler xmlHandler = new XMLHandler(xmlFile);
            Dictionary<String, Object> port = xmlHandler.getPort("example");
            assertEquals("{pins=[GPIO_2], protocol=DI}",port.toString());
        }catch(IOException | ParserConfigurationException | SAXException e){
            System.out.println("XMLHandler failed");
            assert(false);
        }
    }

    @Test
    public void TestInput_ComplexPort(){
        try{
            XMLHandler xmlHandler = new XMLHandler(xmlFile);
            Dictionary<String, Object> port = xmlHandler.getPort("sensorarray");
            assertEquals("{pins=[GPIO_3, GPIO_4, GPIO_5, GPIO_6], protocol=DI}",port.toString());
        }catch(IOException | ParserConfigurationException | SAXException e){
            System.out.println("XMLHandler failed");
            assert(false);
        }
    }
}
